﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.SignalR;
using Microsoft.Extensions.Configuration;
using Newtonsoft.Json;
using Webhook_Catcher.Hub;
using Webhook_Catcher.Models.Home;
using Webhook_Catcher.Models.Webhook;

namespace Webhook_Catcher.Controllers
{
    public class WebhookController : Controller
    {
        private readonly IConfiguration _configuration;
        private static readonly log4net.ILog Log = log4net.LogManager.GetLogger(System.Reflection.MethodBase.GetCurrentMethod()?.DeclaringType);
        private readonly IHubContext<WebhookNotificationHub> _hubContext;
        private readonly IUserManager _userManager;

        public WebhookController(IConfiguration configuration, IHubContext<WebhookNotificationHub> hubcontext, IUserManager userManager)
        {
            _configuration = configuration;
            _hubContext = hubcontext;
            _userManager = userManager;
        }
        
        public async Task<IActionResult> CatchWebhook(int id)
        {
            //logging the data of the processed document
            if (id <= 0)
            {
                Log.Error($"Webhook przysłał zapytanie bez id" );
            }
            else
            {
                Log.Error($"Dokument o id {id.ToString()} został przetworzony." );
            }
            
            //sending the webhook to the right connected user
            var targetClients = _userManager.GetConnectedClients().Where(client => client.UploadedDocument == id).ToArray();
            if (targetClients != null)
            {
                foreach (var target in targetClients)
                {
                    if (target.UploadedDocument == id)
                    {
                        await _hubContext.Clients.All.SendAsync("ReceiveWebhook", id);
                    }
                }
            }

            return Ok();
        }

        [HttpGet]
        public IActionResult GetDocumentResult(int id)
        {
            //creating HTTP client to send request for document result
            var client = new HttpClient();
            client.DefaultRequestHeaders.Add("Accept", "application/json");
            
            //downloading the path to OCR API
            string ocrApi = _configuration["OCRRequestAPI:Url"];
            var uri = $"{ocrApi}api/connector/documents/{id}/result";
            
            //adding the bearer token to the request
            client.DefaultRequestHeaders.Authorization = new AuthenticationHeaderValue("Bearer", HttpContext.Session.GetString("token"));
            
            //receiving the response and converting it
            var response = client.GetAsync(uri).Result;
          
            //error view if there is one
            if (response?.IsSuccessStatusCode == false)
            {
                return View("Error", new ErrorViewModel("Downloading document error"));
            }
            
            var contents = response.Content.ReadAsStringAsync().Result;
            var result = JsonConvert.DeserializeObject<DocumentModel>(contents);
            
            //error view if there is one
            if (!String.IsNullOrEmpty(result?.ErrorMessage))
            {
                return View("Error", new ErrorViewModel(result?.ErrorMessage));
            }
            
            if (!response.IsSuccessStatusCode)
            {
                return View("Error", new ErrorViewModel("Operation has failed - session has expired."));
            }
            
            if (result == null)
            {
                return View("Error", new ErrorViewModel("Operation has failed - no access."));
            }

            if (result.ResultFields == null)
            {
                return View("Error", new ErrorViewModel("Operation has failed - no data."));
            }

            //downloading the link for document editing
            result.EditorLink = GetDocumentEditorLink(id);
            
            if (result.TableResults != null) 
            {
                //function preparing the table results for displaying
                result.PreparedTableResults = PrepareTableResults(result);
            }
            
            //logging the document info
            Log.Error($"Dane dokumentu: [ ID: {result?.Id} - Nazwa: {result?.FileName} - Status: {result?.Status} ]");
            
            //returning the view with document result
            return DocumentResult(result);
        }
        
        [HttpGet]
        public string GetDocumentEditorLink(int id)
        {
            //creating HTTP client to send request for editor link
            var client = new HttpClient();
            client.DefaultRequestHeaders.Add("Accept", "application/json");
            
            //downloading the path to OCR API
            string ocrApi = _configuration["OCRRequestAPI:Url"];
            var uri = $"{ocrApi}api/connector/documents/{id}/link";
            
            //adding the bearer token to the request
            client.DefaultRequestHeaders.Authorization = new AuthenticationHeaderValue("Bearer", HttpContext.Session.GetString("token"));
            
            //receiving the response and converting it
            var response = client.GetAsync(uri).Result;
            var contents = response.Content.ReadAsStringAsync().Result;
            var link = JsonConvert.DeserializeObject<string>(contents);

            return link;
        }

        public PreparedTableResultsModel PrepareTableResults(DocumentModel result)
        {
            var preparedTable = new PreparedTableResultsModel();
            preparedTable.Columns = new List<string>();
            preparedTable.Fields = new List<List<string>>();
            var tableResult = result.TableResults[0];
            foreach (var rows in tableResult.Rows)
            {
                foreach (var field in rows.RowFields)
                {
                    if (!preparedTable.Columns.Contains(field.FieldName))
                    {
                        preparedTable.Columns.Add(field.FieldName);
;                   }
                }
            }

            foreach (var rows in tableResult.Rows)
            {
                var row = new List<string>();
                foreach (var column in preparedTable.Columns)
                {
                    var check = rows.RowFields.Where(field => field.FieldName.Contains(column));
                    if (check.Count() == 0) 
                    {
                        row.Add("");
                    }
                    else
                    {
                        row.Add(check.ToList()[0].ResultText);
                    }
                }
                preparedTable.Fields.Add(row);
            }
            
            return preparedTable;
        }
        
        public IActionResult DocumentResult(DocumentModel content)
        {
            return View("DocumentResult", content);
        }
        
        [HttpGet]
        public IActionResult GetDocumentContent(int id)
        {
            //function that downloads the file

            //creating HTTP client to send a request for document content
            var client = new HttpClient();
            
            //downloading the path to OCR API
            string ocrApi = _configuration["OCRRequestAPI:Url"];
            var uri = $"{ocrApi}api/connector/documents/{id}/content";
            
            //adding the bearer token to the request
            client.DefaultRequestHeaders.Authorization = new AuthenticationHeaderValue("Bearer", HttpContext.Session.GetString("token"));
            
            //receiving the response and converting it
            var response = client.GetAsync(uri).Result;
            var contents = response.Content.ReadAsByteArrayAsync().Result;
            
            //error view if there is one
            if (!response.IsSuccessStatusCode)
            {
                return View("Error", new ErrorViewModel("Operation has failed - document error."));
            }
            
            //returning the file to download
            Response.Headers.Add("Content-Disposition", $"inline; filename=result{id}.pdf");
            return File(contents, "application/pdf", $"result{id}.pdf");
        }

    }
}